from models import TrafficLight
from helpers import handle_input, run_cycles


def main():

    print " "
    print " "
    print "################"
    print " "
    print "Welcome to the Traffic Light Simulator!"
    print "If at any time you'd like to exit just type 'exit'"
    print " "
    print "################"
    print " "
    print " "

    while True:

        try:
            print ("Remember, if you want to quit just type 'exit'\n"
                   "To run infinite cycles you can enter 'forever'\n"
                   "If you want to stop the cycles please use ctrl+c\n"
                   "--------------------------------")
            cycles = handle_input("How many cycles of lights would you like to run? ",
                                  outputtype='int')
            green = handle_input("How many seconds should a green light last? ")
            yellow = handle_input("How many seconds should a yellow light last? ")
            red = handle_input("How many seconds should a red light last? ")

            light = TrafficLight(initial_color='green', 
                                 green_time=green, 
                                 yellow_time=yellow, 
                                 red_time=red)
            run_cycles(light, cycles)
        except (KeyboardInterrupt, EOFError):
            print " "


    print " "
    print " "
    print "################"
    print " "
    print "Exiting Traffic Light Simulator. Goodbye!"
    print " "
    print "################"
    print " "
    print " "

if __name__ == "__main__":
    main()



import sys
import time


def cycle(tlight):
    """
    displays the current_color_art for a TrafficLight object
      for the duration of current_color_time
    :param tlight: TrafficLight object
    """
    seconds = tlight.get_current_color_time()
    while seconds > 0:
        print chr(27) + "[2J"
        print " ~ " + str(seconds)
        print tlight.get_current_art()
        if seconds >= 1:
            time.sleep(1)
        elif seconds >= 0:
            time.sleep(seconds)
        seconds -= 1
    tlight.switch_light()


def run_cycles(tlight, cycles):
    """
    runs a certain number of cycles
    :param tlight: TrafficLight object
    :param cycles: number of cycles you want to run
    """
    if cycles == 'forever':
        while True:
            cycle(tlight)
    else:
        cycles *= 3
        while cycles > 0:
            cycle(tlight)
            cycles -= 1


def handle_input(prompt, outputtype='float'):
    """
    Handles user input, if 'exit' is typed exits program.
    If a non-float is entered it asks user to re-enter input
    :param prompt: string of text for the user prompt
    :param outputtype: a string specifying whether desired datatype of output is int or float
    """
    user_input = raw_input(prompt)
    while True:
        if user_input == 'exit':
            print "Exiting Traffic Light Simulator. Goodbye!"
            sys.exit()
        elif outputtype == 'int' and user_input == 'forever':
            return 'forever'
        try:
            output = float(user_input)
            if output > 0:
                return output
            else:
                raise ValueError('Negative numbers are not allowed')
        except ValueError:
            print "Whoops! That's not right! Please try again."
            user_input = raw_input(prompt)



import os

PACKAGE_ROOT = os.path.abspath(os.path.dirname(__file__))


class TrafficLight():
    """ Traffic Light object to load artwork and keep track of state """

    def __init__(self, initial_color='red', red_time=1, yellow_time=1, green_time=1):
        """
        load artwork and set initial color
        :param initial_color: sets initial value for current_color color property
        :param red_time: sets amount of time for a red light to last
        :param yellow_time: sets amount of time for a yello light to last
        :param green_time: sets amount of time for a green light to last
        """
        red = open(PACKAGE_ROOT + '/light_art/red.txt', 'r').read()
        yellow = open(PACKAGE_ROOT + '/light_art/yellow.txt', 'r').read()
        green = open(PACKAGE_ROOT + '/light_art/green.txt', 'r').read()
        self.light_art = {
            'red': red,
            'yellow': yellow,
            'green': green
        }
        self.light_time = {
            'red': red_time,
            'yellow': yellow_time,
            'green': green_time
        }
        self.current_color = initial_color

    def switch_light(self):
        """ sets current_color to next color in the sequence """
        if self.current_color == 'red':
            self.current_color = 'green'
        elif self.current_color == 'green':
            self.current_color = 'yellow'
        elif self.current_color == 'yellow':
            self.current_color = 'red'

    def get_current_art(self):
        """ returns art for the current color """
        return self.light_art[self.current_color]

    def get_current_color(self):
        """ returns text representation of the current color """
        return self.current_color

    def get_current_color_time(self):
        """ returns the time for the current color """
        return self.light_time[self.current_color]



from distutils.core import setup
from setuptools import find_packages

setup(
    name = 'traffic-light-simulator',
    packages = find_packages(), 
    include_package_data = True,
    package_data = {'traffic_light_simulator': ['*']},
    version = '0.1.2',
    description = 'A Traffic Light Simulator',
    author = 'Caleb Hayashida',
    author_email = 'santeyio@gmail.com',
    url = 'https://github.com/santeyio/traffic-light-simulator', 
    download_url = 'https://github.com/santeyio/traffic-light-simulator/archive/0.1.tar.gz', 
    keywords = ['testing', 'example'], 
    classifiers = [],
    entry_points = {
        'console_scripts': ['traffic-light-simulator=traffic_light_simulator.user_interface:main'],
    },
)



import unittest
from models import TrafficLight


class TrafficLightTestCase(unittest.TestCase):
    
    def test_if_green_light_art_set(self):
        light = TrafficLight()
        self.assertIsNotNone(light.light_art['green'])

    def test_if_yellow_light_art_set(self):
        light = TrafficLight()
        self.assertIsNotNone(light.light_art['yellow'])

    def test_if_red_light_art_set(self):
        light = TrafficLight()
        self.assertIsNotNone(light.light_art['red'])

    def test_if_green_light_time_set(self):
        light = TrafficLight(green_time=2)
        self.assertEqual(light.light_time['green'], 2)

    def test_if_yellow_light_time_set(self):
        light = TrafficLight(yellow_time=3)
        self.assertEqual(light.light_time['yellow'], 3)

    def test_if_red_light_time_set(self):
        light = TrafficLight(red_time=1)
        self.assertEqual(light.light_time['red'], 1)

    def test_if_red_goes_to_green(self):
        light = TrafficLight(initial_color='red')
        light.switch_light()
        self.assertEqual(light.current_color, 'green')

    def test_if_green_goes_to_yellow(self):
        light = TrafficLight(initial_color='green')
        light.switch_light()
        self.assertEqual(light.current_color, 'yellow')

    def test_if_yellow_goes_to_red(self):
        light = TrafficLight(initial_color='yellow')
        light.switch_light()
        self.assertEqual(light.current_color, 'red')
    
    def test_if_get_current_art_returns_something(self):
        light = TrafficLight(initial_color='green')
        current_color = light.get_current_art()
        self.assertIsNotNone(current_color)

    def test_if_get_current_color_returns_green_after_initializing_green(self):
        light = TrafficLight(initial_color='green')
        current_color = light.get_current_color()
        self.assertEqual(current_color, 'green')

    def test_if_get_current_color_time_returns_correct_time(self):
        light = TrafficLight(initial_color='green', green_time=3)
        current_color_time = light.get_current_color_time()
        self.assertEqual(current_color_time, 3)


if __name__ == '__main__':
    unittest.main()
